@@ -29,36 +29,35 @@ class ChallengeFactoryTest(unittest.TestCase):
         # Account is mocked...
         self.handler = AuthHandler(None, None, mock.Mock(key="mock_key"), [])
 
-        self.dom = "test"
-        self.handler.authzr[self.dom] = acme_util.gen_authzr(
-            messages.STATUS_PENDING, self.dom, acme_util.CHALLENGES,
+        self.authzr = acme_util.gen_authzr(
+            messages.STATUS_PENDING, "test", acme_util.CHALLENGES,
             [messages.STATUS_PENDING] * 6, False)
 
     def test_all(self):
         achalls = self.handler._challenge_factory(
-            self.dom, range(0, len(acme_util.CHALLENGES)))
+            self.authzr, range(0, len(acme_util.CHALLENGES)))
 
         self.assertEqual(
             [achall.chall for achall in achalls], acme_util.CHALLENGES)
 
     def test_one_tls_sni(self):
-        achalls = self.handler._challenge_factory(self.dom, [1])
+        achalls = self.handler._challenge_factory(self.authzr, [1])
 
         self.assertEqual(
             [achall.chall for achall in achalls], [acme_util.TLSSNI01])
 
     def test_unrecognized(self):
-        self.handler.authzr["failure.com"] = acme_util.gen_authzr(
-            messages.STATUS_PENDING, "failure.com",
-            [mock.Mock(chall="chall", typ="unrecognized")],
-            [messages.STATUS_PENDING])
+        authzr = acme_util.gen_authzr(
+             messages.STATUS_PENDING, "test",
+             [mock.Mock(chall="chall", typ="unrecognized")],
+             [messages.STATUS_PENDING])
 
         self.assertRaises(
-            errors.Error, self.handler._challenge_factory, "failure.com", [0])
+             errors.Error, self.handler._challenge_factory, authzr, [0])
 
 
-class GetAuthorizationsTest(unittest.TestCase):
-    """get_authorizations test.
+class HandleAuthorizationsTest(unittest.TestCase):
+    """handle_authorizations test.
 
     This tests everything except for all functions under _poll_challenges.
 
@@ -81,6 +80,7 @@ class GetAuthorizationsTest(unittest.TestCase):
 
         self.mock_account = mock.Mock(key=util.Key("file_path", "PEM"))
         self.mock_net = mock.MagicMock(spec=acme_client.Client)
+        self.mock_net.acme_version = 1
 
         self.handler = AuthHandler(
             self.mock_auth, self.mock_net, self.mock_account, [])
@@ -90,20 +90,19 @@ class GetAuthorizationsTest(unittest.TestCase):
     def tearDown(self):
         logging.disable(logging.NOTSET)
 
-    @mock.patch("certbot.auth_handler.AuthHandler._poll_challenges")
-    def test_name1_tls_sni_01_1(self, mock_poll):
-        self.mock_net.request_domain_challenges.side_effect = functools.partial(
-            gen_dom_authzr, challs=acme_util.CHALLENGES)
+    def _test_name1_tls_sni_01_1_common(self, combos):
+        authzr = gen_dom_authzr(domain="0", challs=acme_util.CHALLENGES, combos=combos)
+        mock_order = mock.MagicMock(authorizations=[authzr])
 
-        mock_poll.side_effect = self._validate_all
-
-        authzr = self.handler.get_authorizations(["0"])
+        with mock.patch("certbot.auth_handler.AuthHandler._poll_challenges") as mock_poll:
+            mock_poll.side_effect = self._validate_all
+            authzr = self.handler.handle_authorizations(mock_order)
 
         self.assertEqual(self.mock_net.answer_challenge.call_count, 1)
 
         self.assertEqual(mock_poll.call_count, 1)
-        chall_update = mock_poll.call_args[0][0]
-        self.assertEqual(list(six.iterkeys(chall_update)), ["0"])
+        chall_update = mock_poll.call_args[0][1]
+        self.assertEqual(list(six.iterkeys(chall_update)), [0])
         self.assertEqual(len(chall_update.values()), 1)
 
         self.assertEqual(self.mock_auth.cleanup.call_count, 1)
@@ -113,22 +112,28 @@ class GetAuthorizationsTest(unittest.TestCase):
 
         self.assertEqual(len(authzr), 1)
 
-    @mock.patch("certbot.auth_handler.AuthHandler._poll_challenges")
-    def test_name1_tls_sni_01_1_http_01_1_dns_1(self, mock_poll):
-        self.mock_net.request_domain_challenges.side_effect = functools.partial(
-            gen_dom_authzr, challs=acme_util.CHALLENGES, combos=False)
+    def test_name1_tls_sni_01_1_acme_1(self):
+        self._test_name1_tls_sni_01_1_common(combos=True)
 
+    def test_name1_tls_sni_01_1_acme_2(self):
+        self.mock_net.acme_version = 2
+        self._test_name1_tls_sni_01_1_common(combos=False)
+
+    @mock.patch("certbot.auth_handler.AuthHandler._poll_challenges")
+    def test_name1_tls_sni_01_1_http_01_1_dns_1_acme_1(self, mock_poll):
         mock_poll.side_effect = self._validate_all
         self.mock_auth.get_chall_pref.return_value.append(challenges.HTTP01)
         self.mock_auth.get_chall_pref.return_value.append(challenges.DNS01)
 
-        authzr = self.handler.get_authorizations(["0"])
+        authzr = gen_dom_authzr(domain="0", challs=acme_util.CHALLENGES, combos=False)
+        mock_order = mock.MagicMock(authorizations=[authzr])
+        authzr = self.handler.handle_authorizations(mock_order)
 
         self.assertEqual(self.mock_net.answer_challenge.call_count, 3)
 
         self.assertEqual(mock_poll.call_count, 1)
-        chall_update = mock_poll.call_args[0][0]
-        self.assertEqual(list(six.iterkeys(chall_update)), ["0"])
+        chall_update = mock_poll.call_args[0][1]
+        self.assertEqual(list(six.iterkeys(chall_update)), [0])
         self.assertEqual(len(chall_update.values()), 1)
 
         self.assertEqual(self.mock_auth.cleanup.call_count, 1)
@@ -140,89 +145,160 @@ class GetAuthorizationsTest(unittest.TestCase):
         self.assertEqual(len(authzr), 1)
 
     @mock.patch("certbot.auth_handler.AuthHandler._poll_challenges")
-    def test_name3_tls_sni_01_3(self, mock_poll):
-        self.mock_net.request_domain_challenges.side_effect = functools.partial(
-            gen_dom_authzr, challs=acme_util.CHALLENGES)
-
+    def test_name1_tls_sni_01_1_http_01_1_dns_1_acme_2(self, mock_poll):
+        self.mock_net.acme_version = 2
         mock_poll.side_effect = self._validate_all
+        self.mock_auth.get_chall_pref.return_value.append(challenges.HTTP01)
+        self.mock_auth.get_chall_pref.return_value.append(challenges.DNS01)
 
-        authzr = self.handler.get_authorizations(["0", "1", "2"])
+        authzr = gen_dom_authzr(domain="0", challs=acme_util.CHALLENGES, combos=False)
+        mock_order = mock.MagicMock(authorizations=[authzr])
+        authzr = self.handler.handle_authorizations(mock_order)
+
+        self.assertEqual(self.mock_net.answer_challenge.call_count, 1)
+
+        self.assertEqual(mock_poll.call_count, 1)
+        chall_update = mock_poll.call_args[0][1]
+        self.assertEqual(list(six.iterkeys(chall_update)), [0])
+        self.assertEqual(len(chall_update.values()), 1)
+
+        self.assertEqual(self.mock_auth.cleanup.call_count, 1)
+        cleaned_up_achalls = self.mock_auth.cleanup.call_args[0][0]
+        self.assertEqual(len(cleaned_up_achalls), 1)
+        self.assertEqual(cleaned_up_achalls[0].typ, "tls-sni-01")
+
+        # Length of authorizations list
+        self.assertEqual(len(authzr), 1)
+
+    def _test_name3_tls_sni_01_3_common(self, combos):
+        self.mock_net.request_domain_challenges.side_effect = functools.partial(
+            gen_dom_authzr, challs=acme_util.CHALLENGES, combos=combos)
+
+
+        authzrs = [gen_dom_authzr(domain="0", challs=acme_util.CHALLENGES),
+                   gen_dom_authzr(domain="1", challs=acme_util.CHALLENGES),
+                   gen_dom_authzr(domain="2", challs=acme_util.CHALLENGES)]
+        mock_order = mock.MagicMock(authorizations=authzrs)
+        with mock.patch("certbot.auth_handler.AuthHandler._poll_challenges") as mock_poll:
+            mock_poll.side_effect = self._validate_all
+            authzr = self.handler.handle_authorizations(mock_order)
 
         self.assertEqual(self.mock_net.answer_challenge.call_count, 3)
 
         # Check poll call
         self.assertEqual(mock_poll.call_count, 1)
-        chall_update = mock_poll.call_args[0][0]
+        chall_update = mock_poll.call_args[0][1]
         self.assertEqual(len(list(six.iterkeys(chall_update))), 3)
-        self.assertTrue("0" in list(six.iterkeys(chall_update)))
-        self.assertEqual(len(chall_update["0"]), 1)
-        self.assertTrue("1" in list(six.iterkeys(chall_update)))
-        self.assertEqual(len(chall_update["1"]), 1)
-        self.assertTrue("2" in list(six.iterkeys(chall_update)))
-        self.assertEqual(len(chall_update["2"]), 1)
+        self.assertTrue(0 in list(six.iterkeys(chall_update)))
+        self.assertEqual(len(chall_update[0]), 1)
+        self.assertTrue(1 in list(six.iterkeys(chall_update)))
+        self.assertEqual(len(chall_update[1]), 1)
+        self.assertTrue(2 in list(six.iterkeys(chall_update)))
+        self.assertEqual(len(chall_update[2]), 1)
 
         self.assertEqual(self.mock_auth.cleanup.call_count, 1)
 
         self.assertEqual(len(authzr), 3)
 
+    def test_name3_tls_sni_01_3_common_acme_1(self):
+        self._test_name3_tls_sni_01_3_common(combos=True)
+
+    def test_name3_tls_sni_01_3_common_acme_2(self):
+        self.mock_net.acme_version = 2
+        self._test_name3_tls_sni_01_3_common(combos=False)
+
     @mock.patch("certbot.auth_handler.AuthHandler._poll_challenges")
     def test_debug_challenges(self, mock_poll):
         zope.component.provideUtility(
             mock.Mock(debug_challenges=True), interfaces.IConfig)
-        self.mock_net.request_domain_challenges.side_effect = functools.partial(
-            gen_dom_authzr, challs=acme_util.CHALLENGES)
+        authzrs = [gen_dom_authzr(domain="0", challs=acme_util.CHALLENGES)]
+        mock_order = mock.MagicMock(authorizations=authzrs)
 
         mock_poll.side_effect = self._validate_all
 
-        self.handler.get_authorizations(["0"])
+        self.handler.handle_authorizations(mock_order)
 
         self.assertEqual(self.mock_net.answer_challenge.call_count, 1)
         self.assertEqual(self.mock_display.notification.call_count, 1)
 
     def test_perform_failure(self):
-        self.mock_net.request_domain_challenges.side_effect = functools.partial(
-            gen_dom_authzr, challs=acme_util.CHALLENGES)
+        authzrs = [gen_dom_authzr(domain="0", challs=acme_util.CHALLENGES)]
+        mock_order = mock.MagicMock(authorizations=authzrs)
+
         self.mock_auth.perform.side_effect = errors.AuthorizationError
 
         self.assertRaises(
-            errors.AuthorizationError, self.handler.get_authorizations, ["0"])
+            errors.AuthorizationError, self.handler.handle_authorizations, mock_order)
 
     def test_no_domains(self):
-        self.assertRaises(errors.AuthorizationError, self.handler.get_authorizations, [])
+        mock_order = mock.MagicMock(authorizations=[])
+        self.assertRaises(errors.AuthorizationError, self.handler.handle_authorizations, mock_order)
 
-    @mock.patch("certbot.auth_handler.AuthHandler._poll_challenges")
-    def test_preferred_challenge_choice(self, mock_poll):
-        self.mock_net.request_domain_challenges.side_effect = functools.partial(
-            gen_dom_authzr, challs=acme_util.CHALLENGES)
+    def _test_preferred_challenge_choice_common(self, combos):
+        authzrs = [gen_dom_authzr(domain="0", challs=acme_util.CHALLENGES, combos=combos)]
+        mock_order = mock.MagicMock(authorizations=authzrs)
 
-        mock_poll.side_effect = self._validate_all
         self.mock_auth.get_chall_pref.return_value.append(challenges.HTTP01)
 
         self.handler.pref_challs.extend((challenges.HTTP01.typ,
                                          challenges.DNS01.typ,))
 
-        self.handler.get_authorizations(["0"])
+        with mock.patch("certbot.auth_handler.AuthHandler._poll_challenges") as mock_poll:
+            mock_poll.side_effect = self._validate_all
+            self.handler.handle_authorizations(mock_order)
 
         self.assertEqual(self.mock_auth.cleanup.call_count, 1)
         self.assertEqual(
             self.mock_auth.cleanup.call_args[0][0][0].typ, "http-01")
 
-    def test_preferred_challenges_not_supported(self):
-        self.mock_net.request_domain_challenges.side_effect = functools.partial(
-            gen_dom_authzr, challs=acme_util.CHALLENGES)
+    def test_preferred_challenge_choice_common_acme_1(self):
+        self._test_preferred_challenge_choice_common(combos=True)
+
+    def test_preferred_challenge_choice_common_acme_2(self):
+        self.mock_net.acme_version = 2
+        self._test_preferred_challenge_choice_common(combos=False)
+
+    def _test_preferred_challenges_not_supported_common(self, combos):
+        authzrs = [gen_dom_authzr(domain="0", challs=acme_util.CHALLENGES, combos=combos)]
+        mock_order = mock.MagicMock(authorizations=authzrs)
         self.handler.pref_challs.append(challenges.HTTP01.typ)
         self.assertRaises(
-            errors.AuthorizationError, self.handler.get_authorizations, ["0"])
+            errors.AuthorizationError, self.handler.handle_authorizations, mock_order)
 
-    def _validate_all(self, unused_1, unused_2):
-        for dom in six.iterkeys(self.handler.authzr):
-            azr = self.handler.authzr[dom]
-            self.handler.authzr[dom] = acme_util.gen_authzr(
+    def test_preferred_challenges_not_supported_acme_1(self):
+        self._test_preferred_challenges_not_supported_common(combos=True)
+
+    def test_preferred_challenges_not_supported_acme_2(self):
+        self.mock_net.acme_version = 2
+        self._test_preferred_challenges_not_supported_common(combos=False)
+
+    def test_dns_only_challenge_not_supported(self):
+        authzrs = [gen_dom_authzr(domain="0", challs=[acme_util.DNS01])]
+        mock_order = mock.MagicMock(authorizations=authzrs)
+        self.assertRaises(
+            errors.AuthorizationError, self.handler.handle_authorizations, mock_order)
+
+    def test_perform_error(self):
+        self.mock_auth.perform.side_effect = errors.AuthorizationError
+
+        authzr = gen_dom_authzr(domain="0", challs=acme_util.CHALLENGES, combos=True)
+        mock_order = mock.MagicMock(authorizations=[authzr])
+        self.assertRaises(errors.AuthorizationError, self.handler.handle_authorizations, mock_order)
+
+        self.assertEqual(self.mock_auth.cleanup.call_count, 1)
+        self.assertEqual(
+            self.mock_auth.cleanup.call_args[0][0][0].typ, "tls-sni-01")
+
+    def _validate_all(self, aauthzrs, unused_1, unused_2):
+        for i, aauthzr in enumerate(aauthzrs):
+            azr = aauthzr.authzr
+            updated_azr = acme_util.gen_authzr(
                 messages.STATUS_VALID,
-                dom,
+                azr.body.identifier.value,
                 [challb.chall for challb in azr.body.challenges],
                 [messages.STATUS_VALID] * len(azr.body.challenges),
                 azr.body.combinations)
+            aauthzrs[i] = type(aauthzr)(updated_azr, aauthzr.achalls)
 
 
 class PollChallengesTest(unittest.TestCase):
@@ -231,7 +307,7 @@ class PollChallengesTest(unittest.TestCase):
 
     def setUp(self):
         from certbot.auth_handler import challb_to_achall
-        from certbot.auth_handler import AuthHandler
+        from certbot.auth_handler import AuthHandler, AnnotatedAuthzr
 
         # Account and network are mocked...
         self.mock_net = mock.MagicMock()
@@ -239,40 +315,40 @@ class PollChallengesTest(unittest.TestCase):
             None, self.mock_net, mock.Mock(key="mock_key"), [])
 
         self.doms = ["0", "1", "2"]
-        self.handler.authzr[self.doms[0]] = acme_util.gen_authzr(
-            messages.STATUS_PENDING, self.doms[0],
-            [acme_util.HTTP01, acme_util.TLSSNI01],
-            [messages.STATUS_PENDING] * 2, False)
-
-        self.handler.authzr[self.doms[1]] = acme_util.gen_authzr(
-            messages.STATUS_PENDING, self.doms[1],
-            acme_util.CHALLENGES, [messages.STATUS_PENDING] * 3, False)
-
-        self.handler.authzr[self.doms[2]] = acme_util.gen_authzr(
-            messages.STATUS_PENDING, self.doms[2],
-            acme_util.CHALLENGES, [messages.STATUS_PENDING] * 3, False)
+        self.aauthzrs = [
+            AnnotatedAuthzr(acme_util.gen_authzr(
+                messages.STATUS_PENDING, self.doms[0],
+                [acme_util.HTTP01, acme_util.TLSSNI01],
+                [messages.STATUS_PENDING] * 2, False), []),
+            AnnotatedAuthzr(acme_util.gen_authzr(
+                messages.STATUS_PENDING, self.doms[1],
+                acme_util.CHALLENGES, [messages.STATUS_PENDING] * 3, False), []),
+            AnnotatedAuthzr(acme_util.gen_authzr(
+                messages.STATUS_PENDING, self.doms[2],
+                acme_util.CHALLENGES, [messages.STATUS_PENDING] * 3, False), [])
+        ]
 
         self.chall_update = {}
-        for dom in self.doms:
-            self.chall_update[dom] = [
-                challb_to_achall(challb, mock.Mock(key="dummy_key"), dom)
-                for challb in self.handler.authzr[dom].body.challenges]
+        for i, aauthzr in enumerate(self.aauthzrs):
+            self.chall_update[i] = [
+                challb_to_achall(challb, mock.Mock(key="dummy_key"), self.doms[i])
+                for challb in aauthzr.authzr.body.challenges]
 
     @mock.patch("certbot.auth_handler.time")
     def test_poll_challenges(self, unused_mock_time):
         self.mock_net.poll.side_effect = self._mock_poll_solve_one_valid
-        self.handler._poll_challenges(self.chall_update, False)
+        self.handler._poll_challenges(self.aauthzrs, self.chall_update, False)
 
-        for authzr in self.handler.authzr.values():
-            self.assertEqual(authzr.body.status, messages.STATUS_VALID)
+        for aauthzr in self.aauthzrs:
+            self.assertEqual(aauthzr.authzr.body.status, messages.STATUS_VALID)
 
     @mock.patch("certbot.auth_handler.time")
     def test_poll_challenges_failure_best_effort(self, unused_mock_time):
         self.mock_net.poll.side_effect = self._mock_poll_solve_one_invalid
-        self.handler._poll_challenges(self.chall_update, True)
+        self.handler._poll_challenges(self.aauthzrs, self.chall_update, True)
 
-        for authzr in self.handler.authzr.values():
-            self.assertEqual(authzr.body.status, messages.STATUS_PENDING)
+        for aauthzr in self.aauthzrs:
+            self.assertEqual(aauthzr.authzr.body.status, messages.STATUS_PENDING)
 
     @mock.patch("certbot.auth_handler.time")
     @test_util.patch_get_utility()
@@ -280,21 +356,21 @@ class PollChallengesTest(unittest.TestCase):
         self.mock_net.poll.side_effect = self._mock_poll_solve_one_invalid
         self.assertRaises(
             errors.AuthorizationError, self.handler._poll_challenges,
-            self.chall_update, False)
+            self.aauthzrs, self.chall_update, False)
 
     @mock.patch("certbot.auth_handler.time")
     def test_unable_to_find_challenge_status(self, unused_mock_time):
         from certbot.auth_handler import challb_to_achall
         self.mock_net.poll.side_effect = self._mock_poll_solve_one_valid
-        self.chall_update[self.doms[0]].append(
+        self.chall_update[0].append(
             challb_to_achall(acme_util.DNS01_P, "key", self.doms[0]))
         self.assertRaises(
             errors.AuthorizationError, self.handler._poll_challenges,
-            self.chall_update, False)
+            self.aauthzrs, self.chall_update, False)
 
     def test_verify_authzr_failure(self):
-        self.assertRaises(
-            errors.AuthorizationError, self.handler.verify_authzr_complete)
+        self.assertRaises(errors.AuthorizationError,
+                          self.handler.verify_authzr_complete, self.aauthzrs)
 
     def _mock_poll_solve_one_valid(self, authzr):
         # Pending here because my dummy script won't change the full status.
